///////////////////////////////////////////////////////////////////////////////
// maindlg.h by Jeff Fitzsimons (2002)
//
// This file contains the application's primary dialog interface and all
// state-manipulation logic.
//
// Since this is template-dependent WTL, and VC6 can't handle external
// definitions of template-based code, everything gets stuffed into a
// header file.
///////////////////////////////////////////////////////////////////////////////

// maindlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINDLG_H__4AB33F94_8CD6_4F09_B0B0_8CB3ED29B858__INCLUDED_)
#define AFX_MAINDLG_H__4AB33F94_8CD6_4F09_B0B0_8CB3ED29B858__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Enum.h"
#include "RemoteThread.h"
#include "FireThread.h"
#include <crtdbg.h>

class CMainDlg : public CDialogImpl<CMainDlg>
{
public:
	enum { IDD = IDD_MAINDLG };

	BEGIN_MSG_MAP(CMainDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(IDOK, OnOK)
		COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
		COMMAND_ID_HANDLER(IDC_BTN_REFRESH, OnRefresh)
		COMMAND_ID_HANDLER(IDC_BTN_COMMIT_CHANGES, OnCommitChanges)
		COMMAND_HANDLER(IDC_LST_WINDOWS, LBN_SELCHANGE, OnWindowSelected)
		COMMAND_HANDLER(IDC_LST_WINDOWS, LBN_DBLCLK, OnWindowDoubleClicked)
		COMMAND_HANDLER(IDC_CHK_TOPMOST, BN_CLICKED, OnOptionChecked)
		COMMAND_HANDLER(IDC_CHK_LAYERED, BN_CLICKED, OnOptionChecked)
		COMMAND_HANDLER(IDC_CHK_OPACITY, BN_CLICKED, OnOptionChecked)
		COMMAND_HANDLER(IDC_CHK_TRANSPARENCY, BN_CLICKED, OnOptionChecked)
	END_MSG_MAP()

	CListBox			m_lstWindows;			// A list of all the topmost windows on the system.
	CButton			m_chkTopmost;			// Topmost?
	CButton			m_chkLayered;			// Layered?  (ie Alpha-enabled?)

	CButton			m_chkOpacity;			// Alpha by opacity?
	CRichEditCtrl	m_edtOpacity;			// Opacity level 0..255

	CButton			m_chkTransparency;	// Alpha by transparency?
	CComboBox		m_cmbColor;				// Transparency color

	CButton			m_btnCommitChanges;	// Launches the remote thread on the selected window.
	CStatic			m_stcCount;				// Displays some window-count statistics.

	WNDLIST m_lstRootWindows;

	int m_iSelected;							// Gets set by OnWindowSelected.  LB_ERR if no selection.

	CMainDlg() : m_iSelected(LB_ERR) 
	{ 
	};

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// center the dialog on the screen
		CenterWindow();

		// set icons
		HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
		SetIcon(hIcon, TRUE);
		HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
		SetIcon(hIconSmall, FALSE);

		// Attach all the control wrapper classes...
		m_lstWindows.Attach(GetDlgItem(IDC_LST_WINDOWS));
		m_chkTopmost.Attach(GetDlgItem(IDC_CHK_TOPMOST));
		m_chkLayered.Attach(GetDlgItem(IDC_CHK_LAYERED));

		m_chkOpacity.Attach(GetDlgItem(IDC_CHK_OPACITY));
		m_edtOpacity.Attach(GetDlgItem(IDC_EDT_OPACITY));

		m_chkTransparency.Attach(GetDlgItem(IDC_CHK_TRANSPARENCY));
		m_cmbColor.Attach(GetDlgItem(IDC_CMB_COLOR));

		m_btnCommitChanges.Attach(GetDlgItem(IDC_BTN_COMMIT_CHANGES));
		m_stcCount.Attach(GetDlgItem(IDC_STC_COUNT));

		// I'm only allowing the user to specify three very common colors (see 
		// GetSysColor) to be transparent...
		m_cmbColor.AddString(_T("Button Face"));
		m_cmbColor.AddString(_T("App MDI Workspace"));
		m_cmbColor.AddString(_T("Window"));
		m_cmbColor.SetCurSel(0);

		m_edtOpacity.SetWindowText(_T("128"));

		// SetButtonStates is responsible for enabling/disabling all button 
		// controls based on the value of m_iSelected.
		SetButtonStates();

		// Pop-up the about box.  I threw this in at the end to make sure
		// the user is warned that this is somewhat risky code...
		PostMessage(WM_COMMAND, MAKEWPARAM(ID_APP_ABOUT, BN_CLICKED));
		
		// Enumerate all the windows and populate the window list...
		QueryWindows();

		return TRUE;
	}

	void SetButtonStates()
	{
		BOOL bIsSelectionValid = (m_iSelected != LB_ERR);

		m_chkTopmost.EnableWindow(bIsSelectionValid);
		m_chkLayered.EnableWindow(bIsSelectionValid);
		m_btnCommitChanges.EnableWindow(bIsSelectionValid);

		BOOL bIsTopmost = 
			bIsSelectionValid && 
			( (m_lstRootWindows[m_iSelected].dwStyleEx & WS_EX_TOPMOST) == WS_EX_TOPMOST);

		m_chkTopmost.SetCheck(bIsTopmost);

		BOOL bIsLayered = 
			bIsSelectionValid && 
			( (m_lstRootWindows[m_iSelected].dwStyleEx & WS_EX_LAYERED) == WS_EX_LAYERED);

		m_chkLayered.SetCheck(bIsLayered);

		m_chkOpacity.EnableWindow(bIsLayered);
		m_edtOpacity.EnableWindow(bIsLayered);
		m_chkTransparency.EnableWindow(bIsLayered);
		m_cmbColor.EnableWindow(bIsLayered);
	}

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// Pop up the about box...
		CSimpleDialog<IDD_ABOUTBOX, FALSE> dlg;
		dlg.DoModal();
		return 0;
	}

	LRESULT OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// "Quit" triggers IDOK.
		EndDialog(wID);
		return 0;
	}

	LRESULT OnRefresh(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// The 'Refresh' button was hit:  re-enumerate the windows and populate the
		// window list.
		QueryWindows();
		
		return 0;
	}

	LRESULT OnCommitChanges(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// The changes have been committed, the cached window state is no longer
		// 'dirty'.  (I don't really make use of this, by the way)
		m_lstRootWindows[m_iSelected].bDirty = FALSE;

		WNDINFO *pinfo = &(m_lstRootWindows[m_iSelected]);

		DWORD dwThreadId = 0;
		DWORD dwProcessId = 0;
		
		// Get the process ID of the specified window handle.  We also get the
		// thread ID, but we ignore it...
		dwThreadId = GetWindowThreadProcessId(pinfo->hWnd, &dwProcessId);
		
		// Attempt to open up the target process...
		HANDLE hProcess = OpenProcess(PROCESS_ALL_ACCESS, TRUE, dwProcessId);

		if (hProcess != 0)
		{
			//////////////////////////////////////////////////////////////////////
			// We successfully opened the process.  Now set up the arguments
			// according to the state of our various window controls...

			BOOL bIsLayered = (pinfo->dwStyleEx & WS_EX_LAYERED) == WS_EX_LAYERED;

			DWORD dwLayeredFlags = 0;

			if (m_chkOpacity.GetCheck() & BST_CHECKED)
				dwLayeredFlags |= LWA_ALPHA;

			if (m_chkTransparency.GetCheck() & BST_CHECKED)
				dwLayeredFlags |= LWA_COLORKEY;

			TCHAR arBuffer[10];

			m_edtOpacity.GetWindowText(arBuffer, sizeof(arBuffer));
			INT iOpacity = atoi(arBuffer);

			if (iOpacity < 0)
				iOpacity = 0;
			
			if (iOpacity > 255)
				iOpacity = 255;

			COLORREF crColor;

			switch (m_cmbColor.GetCurSel())
			{
				case 0 : 
					crColor = ::GetSysColor(COLOR_BTNFACE); 
					break;

				case 1 : 
					crColor = ::GetSysColor(COLOR_APPWORKSPACE); 
					break;

				case 2 : 
					crColor = ::GetSysColor(COLOR_WINDOW); 
					break;

				default : 
					crColor = ::GetSysColor(COLOR_BTNFACE); 
					break;
			}

			FireRemoteThread(hProcess, pinfo->hWnd, bIsLayered, dwLayeredFlags, (BYTE)iOpacity, crColor);

			////////////////////////////////////////////////////////////////
			// Set the Z-order of the window (also make the LAYERED changes
			// go into effect).

			BOOL bIsTopmost = (pinfo->dwStyleEx & WS_EX_TOPMOST) == WS_EX_TOPMOST;

			::SetWindowPos(
					pinfo->hWnd, 
					bIsTopmost? HWND_TOPMOST : HWND_NOTOPMOST, 
					0, 0, 0, 0, 
					SWP_NOMOVE | SWP_NOSIZE );

			CloseHandle(hProcess);
		}
		else
		{
			// This snippet of code is stolen from MSDN's FormatMessage() page:
			LPVOID lpMsgBuf;
			FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL 
				);

			TCHAR arBuffer[1024];
			_stprintf(arBuffer, _T("The selected process could not be opened:\n%s"), lpMsgBuf);

			MessageBox(arBuffer, _T("OpenProcess() failure"), MB_OK | MB_ICONERROR);

			LocalFree( lpMsgBuf );
		}

		return 0;
	}

	LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		EndDialog(wID);
		return 0;
	}

	UINT PopulateWindowList(DWORD dwStyleRequire = 0, DWORD dwStyleReject = 0, DWORD dwExStyleRequire = 0, DWORD dwExStyleReject = 0)
	{
		m_lstWindows.ResetContent();

		UINT uiHiddenCount = 0;
		UINT uiChildCount = 0;

		UINT uiCount = 0;

		WNDLIST::iterator i;
		for (i = m_lstRootWindows.begin(); i != m_lstRootWindows.end(); i++)
			if ( 
					((i->dwStyle & dwStyleRequire) == dwStyleRequire)  && 
					((i->dwStyleEx & dwExStyleRequire) == dwExStyleRequire) &&
					!(i->dwStyle & dwStyleReject)  && 
					!(i->dwStyleEx & dwExStyleReject)
				)
			{
				TCHAR arBuffer[1024];
				ZeroMemory(arBuffer, sizeof(arBuffer));

				_stprintf(arBuffer, _T("%s (%s)"), i->strTitle.c_str(), i->strClassname.c_str());

				int iItem = m_lstWindows.AddString(arBuffer);
				m_lstWindows.SetItemData(iItem, i - m_lstRootWindows.begin());

				uiCount++;
			}

		return uiCount;
	}

	VOID QueryWindows()
	{
		m_lstRootWindows.clear();

		::EnumWindows(EnumChildWndCallback, (LONG)&m_lstRootWindows);

		m_lstWindows.ResetContent();

		UINT uiHiddenCount = 0;
		UINT uiChildCount = 0;

		WNDLIST::iterator i;
		for (i = m_lstRootWindows.begin(); i != m_lstRootWindows.end(); i++)
		{
			if ((i->dwStyle & WS_VISIBLE) == 0)
				uiHiddenCount++;

			if (i->dwStyle & WS_CHILD)
				uiChildCount++;
		}

		UINT uiCount = PopulateWindowList(WS_VISIBLE);

		TCHAR arBuffer[255];
		_stprintf(arBuffer,_T("%d of %d windows (%d hidden, %d child)"), uiCount, m_lstRootWindows.size(), uiHiddenCount, uiChildCount);
		m_stcCount.SetWindowText(arBuffer);
	}

	void RetrieveWindowSelection()
	{
		m_iSelected = LB_ERR;

		int iSelected = m_lstWindows.GetCurSel();

		if (iSelected != LB_ERR)
			m_iSelected = m_lstWindows.GetItemData(iSelected);
	}

	LRESULT OnWindowSelected(int Msg, int idCtrl, HWND hWnd, BOOL& bHandled)
	{
		if ( (idCtrl == IDC_LST_WINDOWS) && (Msg == LBN_SELCHANGE) )
		{
			RetrieveWindowSelection();

			SetButtonStates();

			bHandled = TRUE;
		}

		return 0;
	}

	LRESULT OnWindowDoubleClicked(int Msg, int idCtrl, HWND hWnd, BOOL& bHandled)
	{
		if ( (idCtrl == IDC_LST_WINDOWS) && (Msg == LBN_DBLCLK) )
		{
			RetrieveWindowSelection();

			if (m_iSelected != LB_ERR)
				::SetForegroundWindow(m_lstRootWindows[m_iSelected].hWnd);

			SetButtonStates();

			bHandled = TRUE;
		}

		return 0;
	}

	LRESULT OnOptionChecked(int Msg, int idCtrl, HWND hWnd, BOOL& bHandled)
	{
		BOOL bChecked;

		switch (idCtrl)
		{
			case IDC_CHK_TOPMOST :
				bChecked = m_chkTopmost.GetCheck();

				if (bChecked)
					m_lstRootWindows[m_iSelected].dwStyleEx |= WS_EX_TOPMOST;
				else
					m_lstRootWindows[m_iSelected].dwStyleEx &= ~WS_EX_TOPMOST;

				m_lstRootWindows[m_iSelected].bDirty = TRUE;
				break;

			case IDC_CHK_LAYERED :
				bChecked = m_chkLayered.GetCheck();

				if (bChecked)
					m_lstRootWindows[m_iSelected].dwStyleEx |= WS_EX_LAYERED;
				else
					m_lstRootWindows[m_iSelected].dwStyleEx &= ~WS_EX_LAYERED;

				m_chkTransparency.EnableWindow(bChecked);
				m_cmbColor.EnableWindow(bChecked);
				m_chkOpacity.EnableWindow(bChecked);
				m_edtOpacity.EnableWindow(bChecked);

				m_lstRootWindows[m_iSelected].bDirty = TRUE;
				break;

			case IDC_CHK_OPACITY :
				bChecked = m_chkOpacity.GetCheck();

				if (bChecked)
					m_lstRootWindows[m_iSelected].dwStyleEx |= WS_EX_LAYERED;
				else
					m_lstRootWindows[m_iSelected].dwStyleEx &= ~WS_EX_LAYERED;

				m_lstRootWindows[m_iSelected].bDirty = TRUE;
				break;

			case IDC_CHK_TRANSPARENCY :
				bChecked = m_chkTransparency.GetCheck();

				if (bChecked)
					m_lstRootWindows[m_iSelected].dwStyleEx |= WS_EX_LAYERED;
				else
					m_lstRootWindows[m_iSelected].dwStyleEx &= ~WS_EX_LAYERED;

				m_lstRootWindows[m_iSelected].bDirty = TRUE;
				break;
		}
		return 0;
	}
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINDLG_H__4AB33F94_8CD6_4F09_B0B0_8CB3ED29B858__INCLUDED_)
